#!/usr/bin/perl

use strict;

my (
		@fileNames, $fileName, $fileNamesString,
		$timestamp, $threadGroup, $transId, $module, $severity, $message,
		$FirstFPpat, $FirstException, $lineNbr,
		@Exception, $firstAt, $totalExcept,

		%COUNT, %FILELOCATION,
		%MESSAGE, %ERRORIDX, %EXCEPTION, %FIRSTAT, %MSGDETAIL
	    );

sub Dbg {
	my( $line) = @_;

	print "DBG: $line\n";
}

sub numerically { $a <=> $b }

sub printFileLocs {
	my( $errorIdx) = @_;
	my( $i, $idx, $keepGoing);
	my( $fn, $line);

	$keepGoing = 1;
	print "ErrIdx $errorIdx:\n";
	for ( $i = 1; $keepGoing; $i++) {
		$idx = "$errorIdx" . "_$i";
		if ( $FILELOCATION{ $idx} eq undef) {
			$keepGoing = 0;
			next;
		}
		( $fn, $line) = split( /\|/, $FILELOCATION{ $idx});
		print "  in file $fn at line $line\n";
	}
}

sub reportErrors {
	my( $c, $i);
	my( $errorIdx, $count, $exceptionKey, $msg, $exception, $firstat);

	&openDB;
	&iniCount;
	&printFilenames;
	foreach $c ( sort numerically keys %COUNT) {
		$errorIdx = $c;
		$count = $COUNT{ "$errorIdx"};
		print "--------- ErrIdx: $errorIdx; occurred: $count ---------\n";
		$timestamp = $MSGDETAIL{ "$errorIdx" . "_TS"};
		$threadGroup = $MSGDETAIL{ "$errorIdx" . "_TG"};
		$transId = $MSGDETAIL{ "$errorIdx" . "_TI"};
		$module = $MSGDETAIL{ "$errorIdx" . "_MO"};
		$severity = $MSGDETAIL{ "$errorIdx" . "_SE"};
		print "$timestamp\tTG: $threadGroup; TI: $transId; MO: $module; SE: $severity\n";
		$msg = $ERRORIDX{ $errorIdx};
		$msg = substr( $msg, 0, length($msg) - 3);
		print " Msg: $msg\n";
		$i = 1;
		$exceptionKey = "$errorIdx" . "_$i";
		while ( $EXCEPTION{ $exceptionKey} ne undef) {
			$exception = $EXCEPTION{ $exceptionKey};
			print " Exc: $exception\n";
			$i++;
			$exceptionKey = "$errorIdx" . "_$i";
		}
		if ( $FIRSTAT{ $errorIdx} ne undef) {
			$firstat = $FIRSTAT{ $errorIdx};
			print " At:  $firstat\n";
		}
		print "\n";
#LOC		&printFileLocs( $errorIdx);
#LOC		print "\n\n";
	}
	$MESSAGE{ "TotalErrors"} = $totalExcept;
	print "Total errors: $totalExcept\n";
	&closeDB;
}

sub getNextErrorIdx {
	my( $errorIdx);

	if ( $MESSAGE{ "ErrorIndex"} ne undef) {
		$errorIdx = $MESSAGE{ "ErrorIndex"};
	} else {
		$errorIdx = 0;
	}
	$MESSAGE{ "ErrorIndex"} = ++$errorIdx;

	return $errorIdx;
}

sub insertFileLoc {
	my( $errorIdx) = @_;
	my( $i, $lastIdx, $s);

	if ( $FILELOCATION{ "errorIdx"} eq undef) {
		$FILELOCATION{ "errorIdx"} = 0;
	}
	$lastIdx = $FILELOCATION{ "$errorIdx"} += 1;
	$s = "$errorIdx" . "_$lastIdx";
	$FILELOCATION{ $s} = $fileName . "|" . $lineNbr; 
}

sub insertNewError {
	my( $messageKey) = @_;
	my( $errorIdx, $j, $e, $exceptionKey);

	$errorIdx = &getNextErrorIdx;
	$COUNT{ "$errorIdx"} = 1;
	$MESSAGE{ $messageKey} = $errorIdx;
	$ERRORIDX{ $errorIdx} = $messageKey;
	$j = 0;
	foreach $e ( @Exception) {
		$j++;
		$exceptionKey = "$errorIdx" . "_$j";
		$EXCEPTION{ $exceptionKey} = &collapseKey( $e);
	}
	$FIRSTAT{ "$errorIdx"} = &collapseKey( $firstAt);
	$MSGDETAIL{ "$errorIdx" . "_TS"} = $timestamp;
	$MSGDETAIL{ "$errorIdx" . "_TG"} = $threadGroup;
	$MSGDETAIL{ "$errorIdx" . "_TI"} = $transId;
	$MSGDETAIL{ "$errorIdx" . "_MO"} = $module;
	$MSGDETAIL{ "$errorIdx" . "_SE"} = $severity;

#LOC	&insertFileLoc( $errorIdx);

	return( $errorIdx);
}

sub collapseKey {
	my( $inkey) = @_;

	$inkey =~ s/\w{8}-\w{4}-\w{4}-\w{4}-\w{12}/SYSTEMID/; 	# --- SystemId ---
	$inkey =~ s/\(SYSTEMID, \d+\)/(SYSTEMID, LOAD)/;	# --- SID Load ---
	$inkey =~ s/c\d{3}n\d{2}/NODENAME/;		 	# --- Nodename ---
	$inkey =~ s/\d+\.\d+\.\d+\.\d+:\d+/IPPORT/;		# --- IpAddress-Port ---
	$inkey =~ s/\d+\.\d+\.\d+\.\d+/IPADDR/;			# --- IpAddress ---
	$inkey =~ s/[A-Z0-9]{13}[a-z][A-Z0-9]{13}/MD5/;		# --- MD5 ---
	$inkey =~ s/\(\d*%\)/(LIFELINE)/;			# --- Lifeline ---
	$inkey =~ s/[a-z0-9]{27}/LOCKERID/;			# --- LockerId ---

	return $inkey;
}

sub scanError {
	my( $errorIdx, $goAhead, $messageKey, $exceptionKey);
	my( $sameExceptions);
	my( $i, $j, $e);

	$goAhead = 1;
	for ( $i = 1; $goAhead; $i++) {
		$messageKey = $message . " _$i";
		if ( $MESSAGE{ $messageKey} eq undef) {
			$errorIdx = &insertNewError( $messageKey);
#DBG			&Dbg("New error detected $errorIdx");
			$goAhead = 0;
		} else {
			$errorIdx = $MESSAGE{ $messageKey};
			$sameExceptions = 1;
			$j = 0;
			foreach $e ( @Exception) {
				$j++;
				$exceptionKey = "$errorIdx" . "_$j";
				if ( $EXCEPTION{ $exceptionKey} ne undef) {
					if ( $EXCEPTION{ $exceptionKey} ne $e) {
						$sameExceptions = 0;
					}
				} else {
					$sameExceptions = 0;
				}
			}
			$goAhead = !( $sameExceptions);
			if ( !( $goAhead)) {
				if ( $FIRSTAT{ "$errorIdx"} ne undef) {
					if ( $FIRSTAT{ "$errorIdx"} ne $firstAt) {
						$goAhead = 1;
					} else {
						$goAhead = 0;
						$COUNT{ $errorIdx} += 1;
						&insertFileLoc( $errorIdx);
#DBG						&Dbg("Counter for $errorIdx +1");
					}
				}
			}
		}
	}
}

sub identFirstFPat {
	my( $scanLine) = @_;

	if ( $FirstFPpat) {
		if ( $scanLine =~ /^\tat com.filepool/) {
#DBG				print "1_at: $scanLine\n\n";
				$scanLine = &collapseKey( $scanLine);
#DBG				print "1_atC: $scanLine\n\n";
				$firstAt = $scanLine;
				$FirstFPpat = 0;
#DBG				&Dbg("Scan for new error timestamp: $timestamp");
				&scanError;
		}
	}
}

sub identException {
	my( $scanLine) = @_;

	if ( $scanLine =~ /^[\.\w]+Exception/) {
		if ( $FirstException) {
#DBG			print "ts: $timestamp; msg: $message\n";
#DBG			print "tg: $threadGroup; t: $transId; m: $module; s: $severity\n";
			$message = &collapseKey( $message);
#DBG			print "msgC: $message\n";
			$totalExcept++;
			$FirstException = 0;
		}
#DBG		print "E: $scanLine\n";
		$scanLine = &collapseKey( $scanLine);
		push( @Exception, $scanLine);
#DBG		print "EC: $scanLine\n";
		$FirstFPpat = 1;
	}
}

sub parseTimestampLine {
	my( $scanLine) = @_;

	@Exception = ();
	$FirstException = 1;
	( $timestamp, $threadGroup, $transId, $module, $severity, $message) = split( /\t/, $scanLine);
}

sub identTimestamp {
	my( $scanLine) = @_;

	if ( $scanLine =~ /^\d\d\.\d\d\.\d\d\.\d\d\d/) {
		&parseTimestampLine( $scanLine);
	}
}

sub printFilenames {
	my( $i, $keepGoing, $fn);

	$keepGoing = 1;
	print " ------------------ files scanned ------------------\n\n";
	print join("\n", @fileNames);
	#for ($i=1; $keepGoing; $i++) {
		#$fn = "Filename" . "_$i";
		#if ( $MESSAGE{ $fn} eq undef) {
			#$keepGoing = 0;
		#} else {
			#print "Filename: $MESSAGE{ $fn}\n";
		##}
	#}
	print "\n\n\n";
}

sub storeFilename {
	my( $i, $keepGoing, $fn);

	$keepGoing = 1;
	for ($i=1; $keepGoing; $i++) {
		$fn = "Filename" . "_$i";
		if ( $MESSAGE{ $fn} eq undef) {
			$MESSAGE{ $fn} = $fileName;
			$keepGoing = 0;
		}
	}
}

sub scanLog {
	my( $fileName) = @_;
	my( $scanLine);

	if ( $fileName) {
#LOC		$lineNbr = 0;
		if ($fileName =~ /\.gz$/)
		{
		  open( SCAN, "zcat $fileName |") or die "Cannot open file $fileName for read.\n";
		}
		else
		{
		  open( SCAN, "< $fileName") or die "Cannot open file $fileName for read.\n";
		}
		&openDB;
		&iniCount;
		&storeFilename;
		while( chomp( $scanLine = <SCAN>)) {
#LOC			$lineNbr++;
			&identTimestamp( $scanLine);
			&identException( $scanLine);
			&identFirstFPat( $scanLine);
		}
		close( SCAN);
	} else {
		print "Usage: $0 <FileName>\n";
	}
}

sub iniCount {
	#if ( $MESSAGE{ "TotalErrors"} ne undef) {
		#$totalExcept = $MESSAGE{ "TotalErrors"};
	#} else {
		#$totalExcept = 0;
	#}
}

sub openDB {
	dbmopen( %MESSAGE, "./message", 0666) || die "Cannot open hash file MESSAGE.\n";
	dbmopen( %ERRORIDX, "./erroridx", 0666) || die "Cannot open hash file ERRORIDX.\n";
	dbmopen( %EXCEPTION, "./exception", 0666) || die "Cannot open hash file EXCEPTION.\n";
	dbmopen( %FIRSTAT, "./firstat", 0666) || die "Cannot open hash file FIRSTAT.\n";
	dbmopen( %MSGDETAIL, "./msgdetail", 0666) || die "Cannot open hash file MSGDETAIL.\n";
	dbmopen( %COUNT, "./count", 0666) || die "Cannot open hash file COUNT.\n";
#LOC	dbmopen( %FILELOCATION, "./filelocation", 0666) || die "Cannot open hash file FILELOCATION.\n";
#DBG	&Dbg("Opened hashes");
}

sub closeDB {
	dbmclose( %MESSAGE);
	dbmclose( %EXCEPTION);
	dbmclose( %FIRSTAT);
	dbmclose( %MSGDETAIL);
	dbmclose( %COUNT);
#LOC	dbmclose( %FILELOCATION);
}

sub getLogFiles {
        my $directory = shift;
        if (-d $directory)
        {
                local *LOGFILES;
                my $result = opendir(LOGFILES, $directory);
                if ($result)
                {
                        my @files = readdir(LOGFILES);
                        foreach $file (@files)
                        {
                                if (($file ne ".") && ($file ne ".."))
                                {
                                        if (-d $directory . "/" . $file)
                                        {
                                                getLogFiles($directory . "/" . $file);
                                        } else {
                                                if ($file =~ m/FilePool.*/g)
                                                {
                                                        $fileNamesString = $fileNamesString . $directory . "/" . $file . " ";
                                                }
                                        }
                                }
                        }
                        closedir(LOGFILES);
                }
        }
}

# --- Main program --- #
$FirstFPpat = 0;
getLogFiles("/var/local/filepool/Logfiles");
@fileNames = split / /, $fileNamesString;
foreach $fileName (@fileNames)
  {
  &scanLog($fileName);
  &closeDB;
  }
#&reportErrors;
print $totalExcept;
system("rm *.db");
