# Preprocess the FilePool log files, removing unnecessary
# entries from the logs.
#
# This script is intended to be used to preprocess the log
# files before downloading them over a modem line.  By
# removing certain classes of common messages, the amount of
# data to transfer can be dramatically reduced without
# seriously impacting the quality of the log files.

# The rules to use when filtering the logs.
# Each rule has two components, a thread name pattern and
# a message pattern.  If a log entry matches on both patterns,
# then it is considered a "normal" log entry and is excluded
# from the log to be downloaded.
my(@rules) = ( [ "MD5VerifierTask", "^TaskFailed" ],
                       [ "Update Log Processing", "^TaskStarted" ],
               [ "Update Log Processing", "^TaskStopped" ],
                       [ "OrganicRegeneratorTask", "^TaskStarted" ],
                       [ "OrganicRegeneratorTask", "^TaskStopped" ],
                       [ "EventStreamers", "UnknownStatistic" ],
                       [ "Cluster Handlers", "NODE_PARAMSCHANGE.*\\\(online\\\)\$" ] );


                       #check if temporary directory exists, if not create
if (!(-e "/tmp/.centeraviewer"))
{
    mkdir("/tmp/.centeraviewer");
    chmod 0777, "/tmp/.centeraviewer";
}


# Create the directory where the temporary files will be stored
my($PREPROCESSED_LOGS_PREFIX) = "/tmp/.centeraviewer/ZZplp_";
my($current_time) = time;


# Remove any old entries in tmp (more than an hour old is too old)
my(@old_entries) = qx(ls -d $PREPROCESSED_LOGS_PREFIX* 2>/dev/null);
chomp(@old_entries);
my($old_entry);
foreach $old_entry (@old_entries)
    {
    my($prefix, $entry_time) = split(/_/, $old_entry);
    if (($current_time - $entry_time) > 3600)
        {
        #print "Removing old entry $old_entry, time is $entry_time, time now $current_time.\n";
        system("rm -rf $old_entry");
        }
    }


# Make temp preprocessdir!
my($preprocessed_logs_dir) = $PREPROCESSED_LOGS_PREFIX . $current_time;
if (!mkdir($preprocessed_logs_dir))
    {
    print "<command status=\"FAIL\" />\n";
    print "<error value=\"Cannot preprocess logs, use download instead.\" />\n";
    exit(0);
    }

# Get the files to process
my($LOGFILE_LOCATION) = "/var/local/filepool/Logfiles";
my($fileNamesString);
getLogFiles($LOGFILE_LOCATION);

sub getLogFiles {
        my $directory = shift;
        if (-d $directory)
        {
                local *LOGFILES;
                my $result = opendir(LOGFILES, $directory);
                if ($result)
                {
                        my @files = readdir(LOGFILES);
                        foreach $file (@files)
                        {
                                if (($file ne ".") && ($file ne ".."))
                                {
                                        if (-d $directory . "/" . $file)
                                        {
                                                my $dirtomake = $directory . "/" . $file;
                                                $dirtomake =~ s#^$LOGFILE_LOCATION##;
                                                if (!mkdir($preprocessed_logs_dir . $dirtomake))
                                                {
                                                        print "ERROR:Cannot preprocess logs, use download instead........";
                                                        exit(0);

                                                }
                                                getLogFiles($directory . "/" . $file);
                                        } else {
                                                if ($file =~ m/FilePool.*/g)
                                                {
                                                        $fileNamesString = $fileNamesString . $directory . "/" . $file . " ";
                                                }
                                        }
                                }
                        }
                        closedir(LOGFILES);
                }
        }
}

my(@files) = split / /, $fileNamesString;
my($file);




# Preprocess the logs!

foreach $file (@files)
    {
    my($outfile_name) = $file;
    $outfile_name =~ s#^$LOGFILE_LOCATION##;    # Strip off everything but the base filename, like qx(basename)

    if ($file =~ /\.gz$/)
        {
            next if (!open(LOGFILE, "zcat $file |"));
            $outfile_name =~ s/\.gz//;  # Strip off the suffix indicating gzip compression
        }
    else
        {
            next if (!open(LOGFILE, "<$file"));
        }

    if (!open(OUTFILE, ">$preprocessed_logs_dir/$outfile_name"))
        {
            close(LOGFILE);
            next;
        }

    # Process the logfile, only keeping entries which match the filter criteria
    while (<LOGFILE>)
        {
        my($include) = 1;
        # Only exclude lines which are standard log entry lines
        if (/^[0-9]/)
            {
            split('\t');

            my($rule);
            foreach $rule (@rules)
                {
                # $_[1] holds the thread name, $_[5] the log message
                if (($_[1] =~ /$rule->[0]/) && ($_[5] =~ /$rule->[1]/))
                    {
                    $include = 0;
                    last;
                    }
                }
            }

        if ($include)
            {
            print OUTFILE;
            }
        }

    close(LOGFILE);
    close(OUTFILE);
    }


my($tmp_info) = qx (df -k /tmp | grep /tmp);
chomp($tmp_info);
my($tmp_name, $size, $used, $avail, $percent_used, $mount) = split(' ', $tmp_info);
$percent_used =~ s/%$//;                # Remove percentage marker
if ($percent_used > 80)
    {
    print "<command status=\"fail\" />\n";
    print "<error value=\"Insufficient temporary space.\" />\n";
    system("rm -rf $preprocessed_logs_dir");
    exit 0;
    }

#my($gzipped_lines) = qx(zcat $LOGFILE_LOCATION/F*.gz | wc -l);
#my($log_lines) = qx(cat $LOGFILE_LOCATION/F*.log | wc -l);
#my($processed_lines) = qx(cat $preprocessed_logs_dir/* | wc -l);

#chomp($gzipped_lines);
#chomp($log_lines);
#chomp($processed_lines);

#print "Had $gzipped_lines compressed, $log_lines uncompressed, now $processed_lines.\n";
print "<command status=\"OK\" />\n";
print "<result directory=\"$preprocessed_logs_dir\" />\n";


exit(0);

