#
# consolidate.pl
#
# abeloos_ivo@emc.com
#
# 26/11/2002    TTxxxxx integrating subst functionality
# 08/01/2003    TT5838  platform ready - final adjustments
# 12/02/2004    TT11387 handle logfiles in subdirectories
# 12/01/2005    TT16459 script is broken
#

use Time::Local;

my ($FILEPOOL_LOGS)     = "/home/filepool/FP/Logfiles";
my ($OS_LOGS)           = "/var/log";
my ($todayInSeconds)    = time();
my ($periodInSeconds)   = 0;
my ($todayYear)         = 0;
my (%months)            = ("Jan", 0, "Feb", 1, "Mar", 2, "Apr", 3, "May", 4, "Jun", 5, "Jul", 6,
                           "Aug", 7, "Sep", 8, "Oct", 9, "Nov", 10, "Dec", 11);

#
# arguments:
#
#       events: (default ('fp', ('NODE_ONLINE for','NODE_OFFLINE for'), 'os', ())
#               'fp': check in filepool logs
#               'os': check in os logs
#
#       period: (default 4)
#
if (!defined(%events))
{
        %events = (
                                'fp' => ['NODE_ONLINE for','NODE_OFFLINE for'],
                                'os' => ['Mail']
                          );
}
if (!defined($period))
{
        $period = 4;
}

#check if temporary directory exists, if not create
if (!(-e "/tmp/.centeraviewer"))
{
    mkdir("/tmp/.centeraviewer");
    chmod 0777, "/tmp/.centeraviewer";
}

#
# LOGFILE is the file which holds the resulting data
#
my ($resultFile)        = "/tmp/.centeraviewer/consolidate${todayInSeconds}.log";
my ($logFile)           = open(LOGFILE, ">${resultFile}");

#
# Collect all the Filepool logfiles to process
#
my($fileNamesString);
getLogFiles($FILEPOOL_LOGS);

sub getLogFiles {
        my $directory = shift;
        if (-d $directory)
        {
                local *LOGFILES;
                my $result = opendir(LOGFILES, $directory);
                if ($result)
                {
                        local @files = readdir(LOGFILES);
                        foreach $file (@files)
                        {
                                if (($file ne ".") && ($file ne ".."))
                                {
                                        if (-d $directory . "/" . $file)
                                        {
                                                getLogFiles($directory . "/" . $file);
                                        } else {
                                                if ($file =~ m/FilePool.*/g)
                                                {
                                                        $fileNamesString = $fileNamesString . $directory . "/" . $file . " ";
                                                }
                                        }
                                }
                        }
                        closedir(LOGFILES);
                }
        }
}

my(@files) = split / /, $fileNamesString;
#my($file);


#
# consolidate_filepool_logs
#
sub consolidate_filepool_logs
{
        my (@eventsList) = @{$events{'fp'}};
        if ($#eventsList < 0)
        {
                return;
        }

#       if (opendir(LOGS, $FILEPOOL_LOGS))
#       {
#               my (@files) = readdir(LOGS);
                for my $file (@files)
                {
                        #
                        # YYYY.MM.DD
                        #
                        if ($file =~ /(\d\d\d\d)\.(\d\d)\.(\d\d)/)
                        {
                                #
                                # days:         1..31
                                # monthrs:      0..11
                                #
                                my ($fileDate) = timelocal(0, 0, 0, $3, $2 - 1, $1);

                                if (handle_period($fileDate))
                                {
                                        scan_file('fp', $file);
                                }
                        }
                }
#               closedir(LOGS);
#       }
}

#
# consolidate_os_logs
#
sub consolidate_os_logs
{
        my (@eventsList) = @{$events{'os'}};
        if ($#eventsList < 0)
        {
                return;
        }

        if (opendir(LOGS, $OS_LOGS))
        {
                my (@files) = readdir(LOGS);
                for my $file (@files)
                {
                        #
                        # messages
                        #
                        if ($file =~ /^(messages|maillog|sentmail\.log|fp-errors|fp-status|platform\.log|postmortem\.log)/)
                        {
                                my ($command)   = "";
                                my ($lastLine)  = "";

                                if ($file =~ /\.gz$/)
                                {
                                        $command = "zcat $OS_LOGS\/$file \| tail --lines=1";
                                } else {
                                        $command = "tail --lines=1 $OS_LOGS\/$file";
                                }
                                $lastLine = `$command`;

                                if ($lastLine =~ /(^\w+)\s+(\d+)\s+(\d\d\:\d\d\:\d\d) /)
                                {
                                        if (exists $months{$1})
                                        {
                                                #
                                                # days:         1..31
                                                # monthrs:      0..11
                                                #
                                                my ($fileDate) = timelocal(0, 0, 0, $2, $months{$1}, $todayYear);

                                                if (handle_period($fileDate))
                                                {
                                                        scan_file('os', $OS_LOGS . "/" . $file);
                                                }
                                        }
                                }
                        }
                        #
                        #
                        #
                }
                closedir(LOGS);
        }
}

#
# handle_period
#
sub handle_period
{
        my ($compare) = @_;
        my ($difference) = abs($todayInSeconds - $compare);

        return ($difference <= $periodInSeconds);
}

#
# scan_file
#
sub scan_file
{
        my ($type, $fileName) = @_;
        my ($result);

        if (-r $fileName)
        {
                if ($fileName =~ /\.gz$/)
                {
                        $result = open( SCAN, "zcat $fileName |");
                } else {
                        $result = open(SCAN, "<${fileName}");
                }
                if ($result > 0)
                {
                        while (<SCAN>)
                        {
                                my ($line) = $_;
                                foreach my $event (@{$events{$type}})
                                {
                                        if ($event =~ /\((\d+),(\d+)\)(.*)/)
                                        {
                                                my ($start, $stop, $pattern) = ($1, $2, $3);
                                                if ($line =~ /$pattern/)
                                                {
                                                        parse_line($type, $start, $stop, $line);
                                                }
                                        } else {
                                                if ($line =~ /$event/)
                                                {
                                                        parse_line($type, -1, -1, $line);
                                                }
                                        }
                                }
                        }
                        close(SCAN);
                }
        }
}

#
# parse_line
#
sub parse_line
{
        my ($type, $start, $stop, $line) = @_;

        if ($type eq 'fp')
        {
                if ($line =~ /^(\d\d\d\d)\.(\d\d)\.(\d\d).*(\d\d\:\d\d\:\d\d)/)
                {
                        #
                        # FilePool log
                        #
                        my (@lineParts) = split /\t/,$line;

                        #
                        # the subst() works on the message part
                        #
                        my ($message) = $lineParts[$#lineParts];
                        if (($start >= 0) && ($stop >= 0))
                        {
                                $message = substr($message, $start, $stop);
                        }
                        chomp($message);
                        if (length($message) > 0) {
                            print LOGFILE "$1\.$2\.$3 $4\t$lineParts[$#lineParts - 1]\t$message\n";
                        }
                }
        }
        if ($type eq 'os')
        {
                if ($line =~ /(^\w+)\s+(\d+)\s+(\d\d\:\d\d\:\d\d) .*? (.*)/)
                {
                        #
                        # OS log
                        #
                        my ($month)     = 0;
                        if (exists $months{$1})
                        {
                                $month = $months{$1} + 1;
                        }
                        $month = "0${month}";
                        $month = substr($month, length($month) - 2);
                        my ($year) = 1900 + $todayYear;
                        print LOGFILE "$year\.$month\.$2 $3\tSTATUS\t$4\n";
                }
        }
}

#
# initialize
#
sub initialize
{
        #
        #initializes period to 1 day !!
        #
        $periodInSeconds = $period * 24 * 60 * 60;
        ($dummy, $dummy, $dummy, $dummy, $dummy, $todayYear, $dummy, $dummy) = gmtime($todayInSeconds);
}

#
# main()
#

initialize();

my ($fpEvents) = consolidate_filepool_logs();
my ($osEvents) = consolidate_os_logs();

#
# close the opened files
#
close(LOGFILE);
#
# sort and hold the first 2000 lines
#
system("sort -r $resultFile -o $resultFile.sorted");
system("head -n 2000 $resultFile.sorted > $resultFile");

unlink("$resultFile.sorted");

#
# return
#
print $resultFile;

