#
# fp_transfer_get [file=] [split_size=1024] [part_number=1]
#
# abeloos_ivo@emc.com
#

use POSIX;

#
# make sure sensible defaults are used
#
if (!defined(${default_file}))
{
	${default_file} = "";
}
if (!defined(${default_split_size}))
{
	${default_split_size} = 1024;
}
if (!defined(${default_part_number}))
{
	${default_part_number} = 1;
}


#
# default configuration
#
%configuration = (
	file		=> ${default_file},
	split_size	=> ${default_split_size},
	part_number	=> ${default_part_number}
);

#
# command line arguments
#
for ($i=0;$i<=$#ARGV;$i++) {
	foreach $key (keys %configuration) {
		if ($ARGV[$i] =~ /(.*)\=(.*)/) {
			if ($1 eq $key) {
				$configuration{$key} = $2;
			}
		}
	}
}

#
# sanity checks
#
if ($configuration{"split_size"} <= 0)
{
	#
	# NOP
	#
} elsif ($configuration{"part_number"} <= 0)
{
	#
	# NOP
	#
} else {
	#
	# return the number of splits needed
	#
	if (-e $configuration{"file"})
	{
		my $result;

	        $result = open(HANDLE, $configuration{"file"}); 

		if ($result != 0)
		{
			my $bytes	= "";
			my $part_number	= ceil($configuration{"part_number"});	
			my $split_size	= ceil($configuration{"split_size"});	

			my($new_position) = sysseek(HANDLE, ($part_number - 1) * $split_size, 0);
			my($read_len) = sysread(HANDLE, $bytes, $split_size);

			my($encoded_string) = &encode($read_len, $bytes);

			print STDOUT $encoded_string;

			close(HANDLE);
		} else {
			#
			# NOP
			#
		}
	} else {
		#
		# NOP
		#
	}
}


##################################################################
# Perform a Base64 encoding of a byte stream.
# See RFC 1521 for more details of the encoding.
sub encode
{
  my(@base64conversion);
  my($read_len, $input) = @_;
  
  my($value) = 0;
  my($index);
  for ($index = ord('A'); $index <= ord('Z'); $index++, $value++)
  {
      $base64conversion[$value] = chr($index);
  }
  
  for ($index = ord('a'); $index <= ord('z'); $index++, $value++)
  {
      $base64conversion[$value] = chr($index);
  }
  
  for ($index = ord('0'); $index <= ord('9'); $index++, $value++)
  {
      $base64conversion[$value] = chr($index);
  }
  $base64conversion[62] = '+';
  $base64conversion[63] = '/';

  

  my($index);
  my($output, $string);

  for ($index = 0; ($index + 2) < $read_len; $index += 3)
  {
    my($first_byte, $second_byte, $third_byte) = unpack("x${index}C3", $input);

    $output = ($first_byte << 16) + ($second_byte << 8) + ($third_byte);
    $string .= $base64conversion[($output >> 18) & 0x3f] . $base64conversion[($output >> 12) & 0x3f] .
					$base64conversion[($output >> 6) & 0x3f] . $base64conversion[$output & 0x3f];
  }

  if ($index == ($read_len - 2))
  {
     my($first_byte, $second_byte) = unpack("x${index}C2", $input);
     $output = ($first_byte << 16) + ($second_byte << 8);
  
     $string .= $base64conversion[($output >> 18) & 0x3f] . $base64conversion[($output >> 12) & 0x3f] .
					$base64conversion[($output >> 6) & 0x3f] . "=";
  }
  elsif ($index == $read_len - 1)
  {
     my($first_byte) = unpack("x${index}C1", $input);
     $output = ($first_byte << 16);
  
     $string .= $base64conversion[($output >> 18) & 0x3f] . $base64conversion[($output >> 12) & 0x3f] . "==";
  }
  
  
  return($string);
}

