use IO::Socket::INET;

$tempdir = "/tmp/.centeraviewer/";
$logfilename = "${tempdir}download_file.log";

print $logfilename;

#check if temporary directory exists, if not create
if (!(-e $tempdir)) {
    mkdir($tempdir) || die "could not create '$tempdir'";
    chmod(0777, $tempdir);
}

#create the logfile
open(LOGFILE, ">$logfilename") || print "could not create '$logfilename'";
close(LOGFILE);
chmod(0666, $logfilename);

if (!defined($filename)){
  do_error("Required parameter filename not specified.");
}

if (!defined($ipaddress)){
  do_error("Required parameter ipaddress not specified.");
}

if (!defined($ipport)){
  do_error("Required parameter ippport not specified.");
}


if ($filename !~ m#^/#){
  if ($filename =~ m#/#){
    do_error("Non-absolute pathname must not contain directory path: " . $filename);
  }
  else{
    $filename = $tempdir . $filename;
  }
}
log_info("Downloading to " . $filename);


my($CV_socket) = IO::Socket::INET->new(PeerAddr => $ipaddress, PeerPort => $ipport, Proto => "tcp");
if (!$CV_socket){
  do_error("Cannot open socket to " . $ipaddress . ":" . $ipport);
}
log_info("Downloading from " . $ipaddress . ":" . $ipport);

if (-f $filename && ($filename !~ m#/tmp/.centeraviewer#)){
  do_error("$filename already exists - please choose a different name.");
}

if (!open(OUTFILE, ">$filename")){
  do_error("Cannot create file $filename\n");
}

my($data, $data_read);
while (($data_read = sysread($CV_socket, $data, 0x1000)) > 0){
  my($data_written) = syswrite(OUTFILE, $data, $data_read);
  if ($data_written != $data_read){
    do_error("Incomplete file write.  Aborting transfer.");
    unlink($filename);
    break;
    }
}

log_info("Download complete.");

close(OUTFILE);
close($CV_socket);

if (-f $filename){
    log_info("Successful transfer.");
} else {
    do_error("Transfer failed.");
}

close(LOGFILE);

sub do_error {
    log_cat("ERROR", $_[0]);
    exit 1;
}

sub log_info {
    log_cat("INFO", $_[0]);
}

sub log_cat {
    if (open(LOGFILE, ">>$logfilename")){
        ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
        $time = sprintf("%.2d",$mon+1)."/".sprintf("%.2d",$mday)."/".($year + 1900)." ".sprintf("%.2d",$hour).":".sprintf("%.2d",$min).":".sprintf("%.2d",$sec);
        $cat = $_[0];
        $mesg = $_[1];
        print LOGFILE "[$time]: ".sprintf("%-5s", $cat)." $mesg\n";
        close(LOGFILE);
    } else {
        print "error opening '$logfilename'";
    }
}


